@@ -8,11 +8,13 @@
 #include "documentmanager.h"
 #include "favoritelocationmodel.h"
 #include "filehelper.h"
+#include "genericpkpass.h"
 #include "gpxexport.h"
 #include "healthcertificatemanager.h"
 #include "importexport.h"
 #include "livedatamanager.h"
 #include "logging.h"
+#include "passmanager.h"
 #include "pkpassmanager.h"
 #include "reservationmanager.h"
 #include "transfermanager.h"
@@ -48,6 +50,7 @@
 #include <QNetworkAccessManager>
 #include <QNetworkRequest>
 #include <QNetworkReply>
+#include <QScopedValueRollback>
 #include <QStandardPaths>
 #include <QUuid>
 #include <QUrl>
@@ -65,11 +68,7 @@
 #include <kandroidextras/jniarray.h>
 #include <kandroidextras/jnisignature.h>
 #include <kandroidextras/jnitypes.h>
-#include <kandroidextras/manifestpermission.h>
 #include <kandroidextras/uri.h>
-
-#include <QtAndroid>
-#include <QAndroidJniObject>
 #endif
 
 #include <memory>
@@ -78,35 +77,15 @@ using namespace KItinerary;
 
 #ifdef Q_OS_ANDROID
 
-static void importDavDroidJson(JNIEnv *env, jobject that, jstring data)
-{
-    Q_UNUSED(that)
-    const char *str = env->GetStringUTFChars(data, nullptr);
-    const auto doc = QJsonDocument::fromJson(str);
-    env->ReleaseStringUTFChars(data, str);
-
-    const auto array = doc.array();
-    if (array.size() < 2 || array.at(0).toString() != QLatin1String("X-KDE-KITINERARY-RESERVATION")) {
-        return;
-    }
-
-    auto propValue = array.at(1).toString().toUtf8();
-    // work around ical/JSON mis-encoding with newer DAVx⁵ versions
-    propValue.replace("\\,", ",");
-
-    ApplicationController::instance()->importData(propValue);
-}
-
 static void importFromIntent(JNIEnv *env, jobject that, jobject data)
 {
     Q_UNUSED(that)
     Q_UNUSED(env)
-    ApplicationController::instance()->importFromIntent(KAndroidExtras::Intent(data));
+    ApplicationController::instance()->importFromIntent(KAndroidExtras::Jni::fromHandle<KAndroidExtras::Intent>(data));
 }
 
 static const JNINativeMethod methods[] = {
     {"importFromIntent", "(Landroid/content/Intent;)V", (void*)importFromIntent},
-    {"importDavDroidJson", "(Ljava/lang/String;)V", (void*)importDavDroidJson}
 };
 
 Q_DECL_EXPORT jint JNICALL JNI_OnLoad(JavaVM *vm, void*)
@@ -164,7 +143,6 @@ void ApplicationController::setReservationManager(ReservationManager* resMgr)
 void ApplicationController::setPkPassManager(PkPassManager* pkPassMgr)
 {
     m_pkPassMgr = pkPassMgr;
-    connect(m_pkPassMgr, &PkPassManager::passAdded, this, &ApplicationController::importPass);
     connect(m_pkPassMgr, &PkPassManager::passUpdated, this, &ApplicationController::importPass);
 }
 
@@ -193,6 +171,11 @@ void ApplicationController::setTripGroupManager(TripGroupManager *tripGroupMgr)
     m_tripGroupMgr = tripGroupMgr;
 }
 
+void ApplicationController::setPassManager(PassManager *passMgr)
+{
+    m_passMgr = passMgr;
+}
+
 void ApplicationController::setHealthCertificateManager(HealthCertificateManager *healthCertMgr)
 {
     m_healthCertMgr = healthCertMgr;
@@ -351,9 +334,6 @@ void ApplicationController::importLocalFile(const QUrl &url)
     // deal with things we can import more efficiently from a file directly
     const auto head = f.peek(4);
     if (FileHelper::hasZipHeader(head)) {
-        if (url.fileName().endsWith(QLatin1String(".pkpass"), Qt::CaseInsensitive) && !m_pkPassMgr->importPass(url).isEmpty()) {
-            return;
-        }
         if (url.fileName().endsWith(QLatin1String(".itinerary"), Qt::CaseInsensitive) && importBundle(url)) {
             return;
         }
@@ -362,32 +342,29 @@ void ApplicationController::importLocalFile(const QUrl &url)
     importData(f.readAll(), f.fileName());
 }
 
-void ApplicationController::importData(const QByteArray &data, const QString &fileName)
+bool ApplicationController::importData(const QByteArray &data, const QString &fileName)
 {
     qCDebug(Log);
     if (data.size() < 4) {
-        return;
+        return false;
     }
 
     if (FileHelper::hasZipHeader(data)) {
-        if (!m_pkPassMgr->importPassFromData(data).isEmpty()) {
-            return;
-        }
         if (importBundle(data)) {
-            return;
+            return true;
         }
     }
 
+    bool success = false;
     using namespace KItinerary;
     ExtractorEngine engine;
-#if KITINERARY_VERSION >= QT_VERSION_CHECK(5, 19, 41)
     // user opened the file, so we can be reasonably sure they assume it contains
     // relevant content, so try expensive extraction methods too
     engine.setHints(ExtractorEngine::ExtractFullPageRasterImages);
-#endif
     engine.setContextDate(QDateTime(QDate::currentDate(), QTime(0, 0)));
     engine.setData(data, fileName);
-    const auto resIds = m_resMgr->importReservations(JsonLdDocument::fromJson(engine.extract()));
+    const auto extractorResult = JsonLdDocument::fromJson(engine.extract());
+    const auto resIds = m_resMgr->importReservations(extractorResult);
     if (!resIds.isEmpty()) {
         // check if there is a document we want to attach here
         QMimeDatabase db;
@@ -407,38 +384,79 @@ void ApplicationController::importData(const QByteArray &data, const QString &fi
             }
         }
 
+        // collect pkpass files from the document tree
+        QScopedValueRollback importLocker(m_importLock, true);
+        importNode(engine.rootDocumentNode());
+
         Q_EMIT infoMessage(i18np("One reservation imported.", "%1 reservations imported.", resIds.size()));
-        return;
+        success = true;
+    }
+
+    // look for time-less passes/program memberships/etc
+    if (m_passMgr->import(extractorResult) || (resIds.isEmpty() && importGenericPkPass(engine.rootDocumentNode()))) {
+        Q_EMIT infoMessage(i18n("Pass imported."));
+        success = true;
     }
 
     // look for health certificate barcodes instead
     // if we don't find anything, try to import as health certificate directly
     if (importHealthCertificateRecursive(engine.rootDocumentNode()) || m_healthCertMgr->importCertificate(data)) {
         Q_EMIT infoMessage(i18n("Health certificate imported."));
-        return;
+        success = true;
     }
 
     // nothing found
-    Q_EMIT infoMessage(i18n("Nothing imported."));
+    if (!success) {
+        Q_EMIT infoMessage(i18n("Nothing imported."));
+    }
+    return success;
 }
 
-void ApplicationController::checkCalendar()
+bool ApplicationController::importText(const QString& text)
 {
-#ifdef Q_OS_ANDROID
-    using namespace KAndroidExtras;
-    if (QtAndroid::checkPermission(ManifestPermission::READ_CALENDAR) == QtAndroid::PermissionResult::Granted) {
-        const auto activity = QtAndroid::androidActivity();
-        if (activity.isValid()) {
-            activity.callMethod<void>("checkCalendar");
-        }
-    } else {
-        QtAndroid::requestPermissions({ManifestPermission::READ_CALENDAR}, [this] (const QtAndroid::PermissionResultMap &result){
-            if (result[ManifestPermission::READ_CALENDAR] == QtAndroid::PermissionResult::Granted) {
-                checkCalendar();
-            }
-        });
+    return importData(text.toUtf8());
+}
+
+void ApplicationController::importNode(const KItinerary::ExtractorDocumentNode &node)
+{
+    if (node.mimeType() == QLatin1String("application/vnd.apple.pkpass")) {
+        const auto pass = node.content<KPkPass::Pass*>();
+        // ### could we pass along pass directly here to avoid the extra parsing roundtrip?
+        m_pkPassMgr->importPassFromData(pass->rawData());
     }
-#endif
+
+    for (const auto &child : node.childNodes()) {
+        importNode(child);
+    }
+}
+
+bool ApplicationController::importGenericPkPass(const KItinerary::ExtractorDocumentNode &node)
+{
+    if (node.mimeType() == QLatin1String("application/vnd.apple.pkpass")) {
+        const auto pass = node.content<KPkPass::Pass*>();
+        if (pass->type() == KPkPass::Pass::Coupon || pass->type() == KPkPass::Pass::StoreCard) {
+            // no support for displaying those yet
+            return false;
+        }
+
+        GenericPkPass wrapper;
+        wrapper.setName(pass->description());
+        wrapper.setPkpassPassTypeIdentifier(pass->passTypeIdentifier());
+        wrapper.setPkpassSerialNumber(pass->serialNumber());
+        wrapper.setValidUntil(pass->expirationDate());
+
+        QScopedValueRollback importLocker(m_importLock, true);
+        m_pkPassMgr->importPassFromData(pass->rawData());
+        m_passMgr->import(wrapper);
+
+        return true;
+    }
+
+    bool res = false;
+    for (const auto &child : node.childNodes()) {
+        res |= importGenericPkPass(child);
+    }
+    return res;
 }
 
 bool ApplicationController::hasClipboardContent() const
@@ -468,6 +486,7 @@ void ApplicationController::exportToFile(const QUrl &url)
     exporter.exportDocuments(m_docMgr);
     exporter.exportFavoriteLocations(m_favLocModel);
     exporter.exportTransfers(m_resMgr, m_transferMgr);
+    exporter.exportPasses(m_passMgr);
     exporter.exportHealthCertificates(m_healthCertMgr);
     exporter.exportLiveData();
     exporter.exportSettings();
@@ -537,6 +556,7 @@ bool ApplicationController::importBundle(KItinerary::File *file)
         count += importer.importDocuments(m_docMgr);
         count += importer.importFavoriteLocations(m_favLocModel);
         count += importer.importTransfers(m_resMgr, m_transferMgr);
+        count += importer.importPasses(m_passMgr);
         count += importer.importHealthCertificates(m_healthCertMgr);
         count += importer.importLiveData(m_liveDataMgr);
         count += importer.importSettings();
@@ -571,6 +591,10 @@ bool ApplicationController::importHealthCertificateRecursive(const ExtractorDocu
 
 void ApplicationController::importPass(const QString &passId)
 {
+    if (m_importLock) {
+        return;
+    }
+
     const auto pass = m_pkPassMgr->pass(passId);
     KItinerary::ExtractorEngine engine;
     engine.setContent(QVariant::fromValue<KPkPass::Pass*>(pass), u"application/vnd.apple.pkpass");
@@ -634,7 +658,6 @@ void ApplicationController::openDocument(const QUrl &url)
 {
 #ifdef Q_OS_ANDROID
     using namespace KAndroidExtras;
-    auto activity = QtAndroid::androidActivity();
     auto uri = ItineraryActivity().openDocument(url.toLocalFile());
 
     Intent intent;
