@@ -9,10 +9,14 @@
 #include "logging.h"
 
 #include "applicationcontroller.h"
+#include "calendarimportmodel.h"
+#include "countrysubdivisionmodel.h"
 #include "developmentmodecontroller.h"
 #include "documentmanager.h"
 #include "documentsmodel.h"
+#include "factory.h"
 #include "favoritelocationmodel.h"
+#include "genericpkpass.h"
 #include "healthcertificatemanager.h"
 #include "livedatamanager.h"
 #include "localizer.h"
@@ -21,8 +25,10 @@
 #include "mapdownloadmanager.h"
 #include "navigationcontroller.h"
 #include "notificationconfigcontroller.h"
+#include "passmanager.h"
 #include "pkpassmanager.h"
 #include "timelinemodel.h"
+#include "permissionmanager.h"
 #include "pkpassimageprovider.h"
 #include "publictransport.h"
 #include "reservationmanager.h"
@@ -42,7 +48,11 @@
 
 #include <weatherforecastmanager.h>
 
+#include <calendarlistmodel.h>
+#include <calendarpluginloader.h>
+
 #include <KItinerary/CountryDb>
+#include <KItinerary/JsonLdDocument>
 #include <KItinerary/Ticket>
 
 #include <KPkPass/Field>
@@ -51,6 +61,7 @@
 
 #ifndef Q_OS_ANDROID
 #include <KDBusService>
+#include <KWindowSystem>
 #endif
 
 #include <KLocalizedContext>
@@ -60,6 +71,7 @@
 #if HAVE_KCRASH
 #include <KCrash>
 #endif
+#include <KContacts/AddressFormat>
 
 #include <QQuickStyle>
 #include <QQmlApplicationEngine>
@@ -68,9 +80,6 @@
 #ifdef Q_OS_ANDROID
 #include <kandroidextras/activity.h>
 #include <kandroidextras/intent.h>
-
-#include <QtAndroid>
-#include <QAndroidJniObject>
 #else
 #include <QApplication>
 #endif
@@ -82,6 +91,30 @@
 #include <QIcon>
 #include <QWindow>
 
+void registerKContactsTypes()
+{
+    // ### this should move into a real QML plugin for KContacts
+    qmlRegisterUncreatableMetaObject(KContacts::staticMetaObject, "org.kde.contacts", 1, 0, "KContacts", {});
+    qmlRegisterSingletonType("org.kde.contacts", 1, 0, "AddressFormatRepository", [](QQmlEngine *, QJSEngine *jsEngine) -> QJSValue {
+        return jsEngine->toScriptValue(KContacts::AddressFormatRepository());
+    });
+    qRegisterMetaType<KContacts::AddressFormat>();
+    qRegisterMetaType<KContacts::AddressFormatElement>();
+    qRegisterMetaType<KContacts::AddressFormatRepository>();
+    qRegisterMetaType<KContacts::AddressFormatPreference>();
+    qRegisterMetaType<KContacts::AddressFormatScriptPreference>();
+}
+
+void registerKCalendarCoreTypes()
+{
+    // ### this should move into a QML plugin of KCalendarCore
+    qmlRegisterUncreatableMetaObject(KCalendarCore::staticMetaObject, "internal.org.kde.kcalendarcore", 1, 0, "KCalendarCore", {});
+    qmlRegisterType<KCalendarCoreExtras::CalendarListModel>("internal.org.kde.kcalendarcore", 1, 0, "CalendarListModel");
+    qmlRegisterSingletonType("internal.org.kde.kcalendarcore", 1, 0, "CalendarPluginLoader", [](QQmlEngine *, QJSEngine *jsEngine) -> QJSValue {
+        return jsEngine->toScriptValue(KCalendarCoreExtras::CalendarPluginLoader());
+    });
+}
+
 void registerKPkPassTypes()
 {
     qmlRegisterUncreatableType<KPkPass::Barcode>("org.kde.pkpass", 1, 0, "Barcode", {});
@@ -103,6 +136,7 @@ void registerApplicationTypes()
     qRegisterMetaType<Transfer::Alignment>();
     qRegisterMetaType<TripGroupManager*>();
     qRegisterMetaType<WeatherForecast>();
+    qRegisterMetaType<Permission::Permission>();
 
     qmlRegisterUncreatableType<LocationInformation>("org.kde.itinerary", 1, 0, "LocationInformation", {});
     qmlRegisterUncreatableType<StatisticsItem>("org.kde.itinerary", 1, 0, "StatisticsItem", {});
@@ -110,9 +144,12 @@ void registerApplicationTypes()
     qmlRegisterUncreatableType<TimelineModel>("org.kde.itinerary", 1, 0, "TimelineModel", {});
     qmlRegisterUncreatableType<Transfer>("org.kde.itinerary", 1, 0, "Transfer", {});
 
+    qmlRegisterUncreatableMetaObject(Permission::staticMetaObject, "org.kde.itinerary", 1, 0, "Permission", {});
+
+    qmlRegisterType<CalendarImportModel>("org.kde.itinerary", 1, 0, "CalendarImportModel");
+    qmlRegisterType<CountrySubdivisionModel>("org.kde.itinerary", 1, 0, "CountrySubdivisionModel");
     qmlRegisterType<DocumentsModel>("org.kde.itinerary", 1, 0, "DocumentsModel");
     qmlRegisterType<LocationInformationDelegateController>("org.kde.itinerary", 1, 0, "LocationInformationDelegateController");
-    qmlRegisterType<QSortFilterProxyModel>("org.kde.itinerary", 1, 0, "SortFilterProxyModel"); // TODO use this from kitemmodels?
     qmlRegisterType<StatisticsModel>("org.kde.itinerary", 1, 0, "StatisticsModel");
     qmlRegisterType<StatisticsTimeRangeModel>("org.kde.itinerary", 1, 0, "StatisticsTimeRangeModel");
     qmlRegisterType<TicketTokenModel>("org.kde.itinerary", 1, 0, "TicketTokenModel");
@@ -137,6 +174,7 @@ static TripGroupInfoProvider s_tripGroupInfoProvider;
 static TripGroupProxyModel *s_tripGroupProxyModel = nullptr;
 static MapDownloadManager *s_mapDownloadManager = nullptr;
 static HealthCertificateManager *s_healthCertificateManager = nullptr;
+static PassManager *s_passManager = nullptr;
 
 #define REGISTER_SINGLETON_INSTANCE(Class, Instance) \
     qmlRegisterSingletonInstance<Class>("org.kde.itinerary", 1, 0, #Class, Instance);
@@ -167,13 +205,16 @@ void registerApplicationSingletons()
     REGISTER_SINGLETON_INSTANCE(TripGroupProxyModel, s_tripGroupProxyModel)
     REGISTER_SINGLETON_INSTANCE(MapDownloadManager, s_mapDownloadManager)
     REGISTER_SINGLETON_INSTANCE(HealthCertificateManager, s_healthCertificateManager)
+    REGISTER_SINGLETON_INSTANCE(PassManager, s_passManager)
 
     REGISTER_SINGLETON_GADGET_INSTANCE(TripGroupInfoProvider, s_tripGroupInfoProvider)
 
     REGISTER_SINGLETON_GADGET_FACTORY(DevelopmentModeController)
+    REGISTER_SINGLETON_GADGET_FACTORY(Factory)
     REGISTER_SINGLETON_GADGET_FACTORY(Localizer)
     REGISTER_SINGLETON_GADGET_FACTORY(NavigationController)
     REGISTER_SINGLETON_GADGET_FACTORY(NotificationConfigController)
+    REGISTER_SINGLETON_GADGET_FACTORY(PermissionManager)
     REGISTER_SINGLETON_GADGET_FACTORY(PublicTransport)
     REGISTER_SINGLETON_GADGET_FACTORY(Util)
 }
@@ -257,8 +298,8 @@ int main(int argc, char **argv)
     Settings settings;
     s_settings = &settings;
 
-    PkPassManager passMgr;
-    s_pkPassManager = &passMgr;
+    PkPassManager pkPassMgr;
+    s_pkPassManager = &pkPassMgr;
 
     ReservationManager resMgr;
     s_reservationManager = &resMgr;
@@ -274,7 +315,7 @@ int main(int argc, char **argv)
     s_tripGroupManager = &tripGroupMgr;
 
     LiveDataManager liveDataMgr;
-    liveDataMgr.setPkPassManager(&passMgr);
+    liveDataMgr.setPkPassManager(&pkPassMgr);
     liveDataMgr.setReservationManager(&resMgr);
     liveDataMgr.setPollingEnabled(settings.queryLiveData());
     liveDataMgr.setShowNotificationsOnLockScreen(settings.showNotificationOnLockScreen());
@@ -323,14 +364,19 @@ int main(int argc, char **argv)
     HealthCertificateManager healthCertificateMgr;
     s_healthCertificateManager = &healthCertificateMgr;
 
+    KItinerary::JsonLdDocument::registerType<GenericPkPass>();
+    PassManager passMgr;
+    s_passManager = &passMgr;
+
     ApplicationController appController;
     appController.setReservationManager(&resMgr);
-    appController.setPkPassManager(&passMgr);
+    appController.setPkPassManager(&pkPassMgr);
     appController.setDocumentManager(&docMgr);
     appController.setFavoriteLocationModel(&favLocModel);
     appController.setTransferManager(&transferManager);
     appController.setLiveDataManager(&liveDataMgr);
     appController.setTripGroupManager(&tripGroupMgr);
+    appController.setPassManager(&passMgr);
     appController.setHealthCertificateManager(&healthCertificateMgr);
 #ifndef Q_OS_ANDROID
     QObject::connect(&service, &KDBusService::activateRequested, [&](const QStringList &args, const QString &workingDir) {
@@ -341,18 +387,22 @@ int main(int argc, char **argv)
             handleCommandLineArguments(&appController, parser.positionalArguments(), parser.isSet(isTemporaryOpt), parser.value(pageOpt));
         }
         if (!QGuiApplication::allWindows().isEmpty()) {
-            QGuiApplication::allWindows().at(0)->requestActivate();
+            QWindow *window = QGuiApplication::allWindows().at(0);
+            KWindowSystem::updateStartupId(window);
+            KWindowSystem::activateWindow(window);
         }
     });
 #endif
 
+    registerKContactsTypes();
+    registerKCalendarCoreTypes();
     registerKPkPassTypes();
     registerKItineraryTypes();
     registerApplicationTypes();
     registerApplicationSingletons();
 
     QQmlApplicationEngine engine;
-    engine.addImageProvider(QStringLiteral("org.kde.pkpass"), new PkPassImageProvider(&passMgr));
+    engine.addImageProvider(QStringLiteral("org.kde.pkpass"), new PkPassImageProvider(&pkPassMgr));
     auto l10nContext = new KLocalizedContext(&engine);
     l10nContext->setTranslationDomain(QStringLiteral(TRANSLATION_DOMAIN));
     engine.rootContext()->setContextObject(l10nContext);
