@@ -6,6 +6,7 @@
 
 #include "timelinedelegatecontroller.h"
 
+#include "calendarhelper.h"
 #include "constants.h"
 #include "livedatamanager.h"
 #include "locationhelper.h"
@@ -14,8 +15,10 @@
 #include "publictransport.h"
 #include "transfer.h"
 #include "transfermanager.h"
+#include "documentmanager.h"
 
 #include <KItinerary/BusTrip>
+#include <KItinerary/CalendarHandler>
 #include <KItinerary/DocumentUtil>
 #include <KItinerary/Flight>
 #include <KItinerary/LocationUtil>
@@ -129,6 +132,21 @@ void TimelineDelegateController::setTransferManager(QObject *transferMgr)
     Q_EMIT setupChanged();
 }
 
+QObject* TimelineDelegateController::documentManager() const
+{
+    return m_documentMgr;
+}
+
+void TimelineDelegateController::setDocumentManager(QObject *documentMgr)
+{
+    if (m_documentMgr == documentMgr) {
+        return;
+    }
+
+    m_documentMgr = qobject_cast<DocumentManager*>(documentMgr);
+    Q_EMIT setupChanged();
+}
+
 QString TimelineDelegateController::batchId() const
 {
     return m_batchId;
@@ -401,7 +419,7 @@ static bool isLayover(const QVariant &res1, const QVariant &res2)
     return LocationUtil::isSameLocation(LocationUtil::arrivalLocation(res1), LocationUtil::departureLocation(res2), LocationUtil::WalkingDistance);
 }
 
-KPublicTransport::JourneyRequest TimelineDelegateController::journeyRequest() const
+KPublicTransport::JourneyRequest TimelineDelegateController::journeyRequestOne() const
 {
     if (!m_resMgr || m_batchId.isEmpty() || !m_liveDataMgr) {
         return {};
@@ -421,9 +439,18 @@ KPublicTransport::JourneyRequest TimelineDelegateController::journeyRequest() co
     req.setIncludeIntermediateStops(true);
     req.setIncludePaths(true);
     PublicTransport::selectBackends(req, m_liveDataMgr->publicTransportManager(), res);
+    return req;
+}
+
+KPublicTransport::JourneyRequest TimelineDelegateController::journeyRequestFull() const
+{
+    auto req = journeyRequestOne();
+    if (!req.isValid()) {
+        return {};
+    }
 
     // find full journey by looking at subsequent elements
-    auto prevRes = res;
+    auto prevRes = m_resMgr->reservation(m_batchId);
     auto prevBatchId = m_batchId;
     while (true) {
         auto endBatchId = m_resMgr->nextBatch(prevBatchId);
@@ -440,7 +467,7 @@ KPublicTransport::JourneyRequest TimelineDelegateController::journeyRequest() co
     return req;
 }
 
-void TimelineDelegateController::applyJourney(const QVariant &journey)
+void TimelineDelegateController::applyJourney(const QVariant &journey, bool includeFollowing)
 {
     if (!m_resMgr || m_batchId.isEmpty()) {
         return;
@@ -457,7 +484,7 @@ void TimelineDelegateController::applyJourney(const QVariant &journey)
 
     // find all batches we are replying here (same logic as in journeyRequest)
     std::vector<QString> oldBatches({m_batchId});
-    {
+    if (includeFollowing) {
         auto prevRes = m_resMgr->reservation(m_batchId);
         auto prevBatchId = m_batchId;
         while (true) {
@@ -731,11 +758,43 @@ int TimelineDelegateController::documentCount() const
         const auto res = m_resMgr->reservation(resId);
         const auto docIds = DocumentUtil::documentIds(res);
         for (const auto &docId : docIds) {
-            // TODO we could check if actually have the document first?
-            allDocIds.insert(docId.toString());
+            if (m_documentMgr->hasDocument(docId.toString())) {
+                allDocIds.insert(docId.toString());
+            }
         }
     }
     return allDocIds.size();
 }
 
+void TimelineDelegateController::addToCalendar(KCalendarCore::Calendar *cal)
+{
+    const auto resIds = m_resMgr->reservationsForBatch(m_batchId);
+    if (resIds.isEmpty() || !cal) {
+        return;
+    }
+    QVector<QVariant> reservations;
+    reservations.reserve(resIds.size());
+    for (const auto &resId : resIds) {
+        reservations.push_back(m_resMgr->reservation(resId));
+    }
+
+    const auto existingEvents = CalendarHandler::findEvents(cal, reservations.at(0));
+    KCalendarCore::Event::Ptr event;
+    if (existingEvents.size() == 1) {
+        event = existingEvents.at(0);
+        event->startUpdates();
+    } else {
+        event = KCalendarCore::Event::Ptr(new KCalendarCore::Event);
+    }
+
+    CalendarHandler::fillEvent(reservations, event);
+    CalendarHelper::fillPreTransfer(event, m_transferMgr->transfer(m_batchId, Transfer::Before));
+
+    if (existingEvents.size() == 1) {
+        event->endUpdates();
+    } else {
+        cal->addEvent(event);
+    }
+}
+
 #include "moc_timelinedelegatecontroller.cpp"
